#ifndef INCLUDED_ESDOM_Document_hpp
#define INCLUDED_ESDOM_Document_hpp

#include <ESDOM/NodeBase.hpp>
#include <ESDOM/ESDOMException.hpp>

namespace ESDOM
{

class ESDOM_EXPORT DocumentType;
class ESDOM_EXPORT Element;
class ESDOM_EXPORT Text;
class ESDOM_EXPORT Comment;
class ESDOM_EXPORT CDATASection;
class ESDOM_EXPORT DocumentInst;

class ESDOM_EXPORT Document : public virtual NodeBase
{
public:
	Document();
	Document(const Document &original);
	Document(const DocumentInst *instance);
	virtual ~Document();

	virtual Document &	operator = (const Document &node);

	virtual DocumentType	getDoctype(void) const;
	virtual bool			getStandalone(void) const;
	virtual void			setStandalone(bool n);
	virtual Element			getDocumentElement(void) const;
	virtual Element					createElement(const ESDOM_CHAR *tagName) const throw(ESDOMException);
	virtual Text					createTextNode(const ESDOM_CHAR *data) const;
	virtual Comment					createComment(const ESDOM_CHAR *data) const;
	virtual CDATASection			createCDATASection(const ESDOM_CHAR *data) const;
	virtual ProcessingInstruction	createProcessingInstruction(const ESDOM_CHAR *target, const ESDOM_CHAR *data) const throw(ESDOMException);
	virtual Attr					createAttribute(const ESDOM_CHAR *name) const throw(ESDOMException);
	virtual NodeList		getElementsByTagName(const ESDOM_CHAR *name) const;
	virtual Element			getElementById(const ESDOM_CHAR *id) const;
	virtual ESDOMString		setDoctype(const ESDOM_CHAR *doctype);

	static Document _createNewDocument( const ESDOM_CHAR *version,
										const ESDOM_CHAR *rootTagName,
										const ESDOM_CHAR *docutype );
protected:

//	virtual void	_setDocumentElement(Element element);

	friend	class	Parser;
	friend	class	Serializer;

};

}; /* namespace ESDOM */

#endif /* INCLUDED_ESDOM_Document_hpp */
