#ifndef INCLUDED_ESDOM_ESDOMString_hpp
#define INCLUDED_ESDOM_ESDOMString_hpp

namespace ESDOM
{

class ESDOM_EXPORT ESDOMStringInst;

class ESDOM_EXPORT ESDOMString
{
public:
	ESDOMString();
	ESDOMString(const ESDOM_CHAR *string);
	ESDOMString(const char *string);
	ESDOMString(const ESDOMString &original);
	ESDOMString(ESDOM_CHAR ch);
	ESDOMString(int n);
	ESDOMString(unsigned n);
	~ESDOMString();

	operator const ESDOM_CHAR*() const;
	operator bool()const;

	ESDOMString &	operator = (const ESDOMString &string);
	ESDOMString &	operator = (const ESDOM_CHAR * string);
	ESDOMString &	operator = (const char * string);
	ESDOMString &	operator = (ESDOM_CHAR ch);
	ESDOMString &	operator = (int n);
	ESDOMString &	operator = (unsigned n);
	bool			operator == (const ESDOMString &string) const;
	bool			operator == (const ESDOM_CHAR *string) const;
	bool			operator == (const char *string) const;
	bool			operator == (ESDOM_CHAR ch) const;
	bool			operator == (int n) const;
	bool			operator == (unsigned n) const;
	bool			operator != (const ESDOMString &string) const;
	bool			operator != (const ESDOM_CHAR *string) const;
	bool			operator != (const char *string) const;
	bool			operator != (ESDOM_CHAR ch) const;
	bool			operator != (int n) const;
	bool			operator != (unsigned n) const;

	ESDOMString &	operator += (const ESDOMString &string);
	ESDOMString &	operator += (const ESDOM_CHAR *string);
	ESDOMString &	operator += (const char *string);
	ESDOMString &	operator += (ESDOM_CHAR ch);
	ESDOMString &	operator += (int n);
	ESDOMString &	operator += (unsigned n);

	unsigned long	getLength(void) const;
	int				Dec_to_Int(void) const;
	unsigned		Dec_to_UInt(void) const;
	unsigned		Hex_to_UInt(void) const;
	unsigned long	index(const ESDOMString &substring) const;
	unsigned long	index(const ESDOM_CHAR *substring) const;
	ESDOMString		substr(unsigned long index, unsigned long length =(unsigned long)-1) const;

	bool			isName(void) const;
	static bool		isName(const ESDOM_CHAR *string);
	static bool		isNameChar(ESDOM_CHAR ch){
		return ( isLetter(ch) || isDigit(ch)
			|| ch == '.' || ch == '-' || ch == '_' || ch == ':'
			|| isCombiningChar(ch) || isExtender(ch) );
	}
	static bool		isSpace(ESDOM_CHAR ch);
	static bool		isChar(ESDOM_CHAR ch);
	static bool		isLetter(ESDOM_CHAR ch);
	static bool		isBaseChar(ESDOM_CHAR ch);
	static bool		isIdeographic(ESDOM_CHAR ch);
	static bool		isCombiningChar(ESDOM_CHAR ch);
	static bool		isDigit(ESDOM_CHAR ch);
	static bool		isExtender(ESDOM_CHAR ch);


protected:
	ESDOMStringInst	*m_pInst;
	static unsigned short	FIGURE_NUMBER; // for compatible
	static const ESDOM_CHAR	ESDOM_NULL_String[2];

	void		make_from_ESDOM_CHAR(const ESDOM_CHAR *string);
	void		make_from_char(const char *string);
	void		make_from_int(int n);
	void		make_from_uint(unsigned n);


	friend bool ESDOM_EXPORT	operator == (const ESDOM_CHAR *ls,	const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator == (const char *ls,		const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator == (ESDOM_CHAR lch,		const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator == (int ln,				const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator == (unsigned ln,			const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator != (const ESDOM_CHAR *ls,	const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator != (const char *ls,		const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator != (ESDOM_CHAR lch,		const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator != (int ln,				const ESDOMString &rs);
	friend bool ESDOM_EXPORT	operator != (unsigned ln,			const ESDOMString &rs);

	friend	class	Initializer;

};

ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	const ESDOMString &rs);
ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	const ESDOM_CHAR *rs);
ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	const char *rs);
ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	ESDOM_CHAR rch);
ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	int rn);
ESDOMString ESDOM_EXPORT	operator + (const ESDOMString &ls,	unsigned rn);
ESDOMString ESDOM_EXPORT	operator + (const ESDOM_CHAR *ls,	const ESDOMString &rs);
ESDOMString ESDOM_EXPORT	operator + (const char *ls,			const ESDOMString &rs);
ESDOMString ESDOM_EXPORT	operator + (ESDOM_CHAR lch,			const ESDOMString &rs);
ESDOMString ESDOM_EXPORT	operator + (int n,					const ESDOMString &rs);
ESDOMString ESDOM_EXPORT	operator + (unsigned n,				const ESDOMString &rs);


}; /* namespace ESDOM */

#endif /* INCLUDED_ESDOM_ESDOMString_hpp */
