#ifndef INCLUDED_ESDOM_NodeBase_hpp
#define INCLUDED_ESDOM_NodeBase_hpp

#include <ESDOM/ESDOMString.hpp>
#include <ESDOM/ESDOMException.hpp>
#include <ESDOM/Interface.hpp>

namespace ESDOM
{

class ESDOM_EXPORT NodeInst;
class ESDOM_EXPORT NodeList;
class ESDOM_EXPORT NamedNodeMap;

class ESDOM_EXPORT Element;
class ESDOM_EXPORT Attr;
class ESDOM_EXPORT Text;
class ESDOM_EXPORT CDATASection;
class ESDOM_EXPORT ProcessingInstruction;
class ESDOM_EXPORT Comment;
class ESDOM_EXPORT Document;
class ESDOM_EXPORT DocumentType;

class ESDOM_EXPORT Node;

class ESDOM_EXPORT NodeBase : public Interface
{
public:

	NodeBase();
	NodeBase(const NodeBase &original);
	NodeBase(const NodeInst *instance);
	virtual ~NodeBase();

	virtual NodeBase & operator = (const NodeBase &node);

	virtual	ESDOMString		getNodeName(void) const;
	virtual	ESDOMString		getNodeValue(void) const;
	virtual	void			setNodeValue(const ESDOM_CHAR *value) throw(ESDOMException);
	virtual	unsigned short	getNodeType(void) const;
	virtual Node			getParentNode(void) const;
	virtual	NodeList		getChildNodes(void) const;
	virtual	Node			getFirstChild(void) const;
	virtual	Node			getLastChild(void) const;
	virtual	Node			getPreviousSibling(void) const;
	virtual	Node			getNextSibling(void) const;
	virtual	NamedNodeMap	getAttributes(void) const;
	virtual	Node			insertBefore(Node newChild, Node refChild) throw(ESDOMException);
	virtual	Node			replaceChild(Node newChild, Node oldChild) throw(ESDOMException);
	virtual	Node			removeChild(Node oldChild) throw(ESDOMException);
	virtual	Node			appendChild(Node newChild) throw(ESDOMException);
	virtual	bool			hasChildNodes(void) const;
	virtual	Node			cloneNode(bool deep) const;

	virtual operator Element () const;
	virtual operator Attr () const;
	virtual operator Text () const;
	virtual operator CDATASection () const;
	virtual operator ProcessingInstruction () const;
	virtual operator Comment () const;
	virtual operator Document () const;
	virtual operator DocumentType () const;
	virtual operator Node () const;

protected:
};

}; /* namespace ESDOM */

#endif /* INCLUDED_ESDOM_NodeBase_hpp */
